"""General Constants"""
from enum import Enum

URL_LOGIN = "https://www.trading212.com/en/login"
CLASS_COOKIES_NOTICE_BUTTON = "CookiesNotice_button__q5YaL"
CLASS_LOGIN_BUTTON = "SubmitButton_input__IV2dl"
CLASS_EQUITY_ICON = "equity-icon"
CLASS_CFD_ICON = "cfd-icon"
PYTRADING212_VERSION = "0.2.5"

class Mode(Enum):
    """Mode Type"""
    DEMO = "demo",
    LIVE = "live"


class Trading(Enum):
    """Trading Type"""
    CFD = "CFD",
    EQUITY = "EQUITY",


class Period(Enum):
    """Period of Portfolio Performance"""
    LAST_DAY = "LAST_DAY",
    LAST_WEEK = "LAST_WEEK",
    LAST_MONTH = "LAST_MONTH",
    LAST_THREE_MONTHS = "LAST_THREE_MONTHS",
    LAST_YEAR = "LAST_YEAR",
    ALL = "ALL",


class OrderType(Enum):
    """Order Type"""
    LIMIT = "LIMIT"
    STOP = "STOP"
    MARKET = "MARKET"
    STOP_LIMIT = "STOP_LIMIT"


class TimeValidity(Enum):
    """Time Validity for Limit/Stop Orders"""
    DAY = "DAY",
    GOOD_TILL_CANCEL = "GOOD_TILL_CANCEL"



import pytest

from pytrading212 import constants


@pytest.fixture
def test_order():
    from pytrading212 import EquityOrder
    def test_order(equity_order: EquityOrder, expected_json: str):
        assert equity_order.to_json().replace(" ", "") == expected_json.replace(" ", "")

    return test_order


def test_market_order(test_order):
    from pytrading212 import MarketOrder
    test_order(
        equity_order=MarketOrder(instrument_code="AAPL_US_EQ", quantity=1),
        expected_json='{"instrumentCode":"AAPL_US_EQ","orderType":"MARKET","quantity":1}'
    )


def test_limit_order(test_order):
    from pytrading212 import LimitOrder
    test_order(
        equity_order=LimitOrder(instrument_code="AAPL_US_EQ",
                                quantity=2,
                                limit_price=150,
                                time_validity=constants.TimeValidity.DAY),
        expected_json='{"instrumentCode":"AAPL_US_EQ","orderType":"LIMIT",'
                      '"quantity":2,"limitPrice":150,"timeValidity":"DAY"}'
    )


def test_stop_order(test_order):
    from pytrading212 import StopOrder
    test_order(equity_order=StopOrder(instrument_code="AAPL_US_EQ", quantity=1, stop_price=150,
                                      time_validity=constants.TimeValidity.GOOD_TILL_CANCEL),
               expected_json='{"instrumentCode":"AAPL_US_EQ","orderType":"STOP","quantity":1,"stopPrice":150,'
                             '"timeValidity":"GOOD_TILL_CANCEL"}')


def test_stop_limit_order(test_order):
    from pytrading212 import StopLimitOrder
    test_order(equity_order=StopLimitOrder(instrument_code="AAPL_US_EQ",
                                           quantity=1,
                                           limit_price=150,
                                           stop_price=160,
                                           time_validity=constants.TimeValidity.DAY),
               expected_json='{"instrumentCode":"AAPL_US_EQ","orderType":"STOP_LIMIT","quantity":1,"limitPrice":150,'
                             '"stopPrice":160,"timeValidity":"DAY"}')


def test_value_order(test_order):
    from pytrading212 import ValueOrder
    test_order(equity_order=ValueOrder(instrument_code="AAPL_US_EQ", value=2500.0),
               expected_json='{"instrumentCode":"AAPL_US_EQ","orderType":"MARKET","value":2500.0}')



class Company:
    """Company Wrapper"""

    def __init__(self, instrument_code: str, isin: str):
        self.instrument_code = instrument_code
        self.isin = isin


class CFDInstrument:
    def __init__(self, name: str, ticker: str, sell_price: float, buy_price: float):
        self.name = name
        self.ticker = ticker
        self.sell_price = sell_price
        self.buy_price = buy_price



from .order import EquityOrder, MarketOrder, LimitOrder, StopOrder, StopLimitOrder, ValueOrder, CFDMarketOrder, \
    CFDLimitStopOrder, CFDOCOOrder
from .trading212 import Equity, CFD
from .constants import Mode, Trading, Period, OrderType, TimeValidity
from .position import Position
from .instrument import CFDInstrument, Company
